#include "MassPoint.h"


/**
 * The number of points
 */
int MassPoint::numPoints() {
	return 0;
}


/**
 * The dimension of the 'state' array
 */
int MassPoint::getDimension() {
	return 6*numPoints();
}


/**
 * Applys all forces
 */
void MassPoint::applyForces() {
	for (int i = 0; i < forces.getLength(); i++)
		forces[i].applyForce();
}


/**
 * Overrides 'forceAccums' with passed force
 */
void MassPoint::setForceAccumulatorsWithForce(float fx, float fy, float fz) {
	int j = 0;
	int dim = numPoints()*3;
	for (int i = 0; i < dim; i+=3) {
		if (fixed[j]) {
            forceAccums[i + DIM_X] = 0.0;
			forceAccums[i + DIM_Y] = 0.0;
			forceAccums[i + DIM_Z] = 0.0;
		} else {
			forceAccums[i + DIM_X] = fx;
			forceAccums[i + DIM_Y] = fy;
			forceAccums[i + DIM_Z] = fz;
		}
		j++;
	}
}


/**
 * Overrides 'forceAccums' and applys the acceleration
 */
void MassPoint::setForceAccumulatorsWithAcceleration(float ax, float ay, float az) {
	int j = 0;
	int dim = numPoints()*3;
	for (int i = 0; i < dim; i+=3) {
		if (fixed[j]) {
            forceAccums[i + DIM_X] = 0.0;
			forceAccums[i + DIM_Y] = 0.0;
			forceAccums[i + DIM_Z] = 0.0;
		} else {
			forceAccums[i + DIM_X] = - masses[j] * ax;
			forceAccums[i + DIM_Y] = - masses[j] * ay;
			forceAccums[i + DIM_Z] = - masses[j] * az;
		}
		j++;
	}
}


/**
 * Calcualtes the derivatives of the positions and the velocities
 *
 * f = m * a = m * x''
 * a = f / m = x''
 *
 * x' = v
 * v' = a = f / m
 *
 *                         / v[0]             \     / y[numPoints*3]                                 \
 *                        |   ...              |   |    ...                                           |
 *                        |  v[numPoints*3-1]  |   |  y[2*numPoints*3 - 1]                            |
 *  dy/dt = f(y,t)  <==>  |  a[0]              | = |  forceAccums[0]/masses[0]                        |
 *                        |   ...              |   |    ...                                           |
 *                         \ a[numPoints*3-1] /     \ forceAccums[numPoints*3-1]/masses[numPoints-1] /
 */
void MassPoint::calculateDerivatives (float derivatives[]) {
	int ofset = numPoints()*3;
	int j = 0;
	for (int i=0; i < ofset; i+=3) {
		// x[i]' = v[i]
		derivatives[i + DIM_X] = state[ofset + i + DIM_X];
		derivatives[i + DIM_Y] = state[ofset + i + DIM_Y];
		derivatives[i + DIM_Z] = state[ofset + i + DIM_Z];

		// v[i]' = a[i] = f[i]/m[i]
		derivatives[ofset + i + DIM_X] = forceAccums[i + DIM_X] / masses[j];
		derivatives[ofset + i + DIM_Y] = forceAccums[i + DIM_Y] / masses[j];
		derivatives[ofset + i + DIM_Z] = forceAccums[i + DIM_Z] / masses[j];
		j++;
	}
}

void MassPoint::calculateAccelerations(float accelerations[]) {
	int j = 0;
	for (int i = 0; i < numPoints()*3; i+=3) {
		// v[i]' = a[i] = f[i]/m[i]
		accelerations[i + DIM_X] = forceAccums[i + DIM_X] / masses[j];
		accelerations[i + DIM_Y] = forceAccums[i + DIM_Y] / masses[j];
		accelerations[i + DIM_Z] = forceAccums[i + DIM_Z] / masses[j];
		j++;
	}
}


// deferred features
void MassPoint::calculateForces() {}
void MassPoint::step() {}
void ODE_System::calculateDerivatives(float derivatives[]) {}
void ODE_System::calculateAccelerations(float accelerations[]) {}
int ODE_System::getDimension() {return 0;}