#include "Solver.h"
#include <stdio.h>

/**
 * The Vector math stuff for the solver classes
 */
void Solver::scaleVector(float vector[], float factor, int length) {
	for(int i = 0; i < length; i++) {
		vector[i] *= factor;
	}
}
void Solver::addVector(float vector[], float summand[], int length) {
	for(int i = 0; i < length; i++) {
		vector[i] += summand[i];
	}
}
void Solver::addScaledVector(float vector[], float summand[], float factor, int length) {
	for(int i = 0; i < length; i++) {
		vector[i] += factor * summand[i];
	}
}




/**
 * Euler
 */
Euler::Euler(ODE_System *system_, float dt_) {
	odeSystem = system_;
	dt = dt_;
	derivatives = new float[odeSystem->getDimension()];
}

void Euler::performStep() {
	odeSystem->calculateDerivatives(derivatives);
	addScaledVector(odeSystem->state, derivatives, dt, odeSystem->getDimension());
}






/**
 * LeapFrog
 */
LeapFrog::LeapFrog(ODE_System *system_, float dt_) {
	odeSystem = system_;
	dt = dt_;
	derivatives = new float[odeSystem->getDimension()];

    // Initialize the velocity to v(t-dt/2) = v(t) - a(t)*dt/2:
	odeSystem->calculateDerivatives(derivatives);
	int dim = odeSystem->getDimension()/2;
	addScaledVector(&odeSystem->state[dim], &derivatives[dim], -dt/2.0f, dim);
}

void LeapFrog::performStep() {
	int dim = odeSystem->getDimension()/2;
	odeSystem->calculateDerivatives(derivatives);
	addScaledVector(&odeSystem->state[dim], &derivatives[dim], dt, dim);
	addScaledVector(&odeSystem->state[0], &odeSystem->state[dim], dt, dim);
}







/**
 * Beeman
 */
Beeman::Beeman(ODE_System *system_, float dt_) {
	odeSystem = system_; 
	dt = dt_;
	int dim = odeSystem->getDimension();
	position    = true;
	derivatives = new float[dim/2];
	a0          = new float[dim/2];  // a(t-h)
	a1          = new float[dim/2];  // a(t)

	// Initialize the array storings the old accelarations: 
	// a(t-h) = g
	// a(t)   = F(x(t))/m;
	odeSystem->calculateAccelerations(a1);
	for (int i = 0; i < dim/2; i+=3)
	{
		a0[i] = 0.0;
		a0[i + 1] = 0.0;
		a0[i + 2] = 9.0;
	}
}

/**
 * x(t+h) = x(t) + h*v(t) + h*h( 2/3*a(t) - 1/6*a(t-h) )     + O(h^4)
 * v(t+h) = v(t) + h*(5/12*a(t+h) + 2/3*a(t) - 1/12*a(t-h) ) + O(h^4)
 */
void Beeman::performStep() {
	int dim = odeSystem->getDimension()/2;
	if (position) {
        addScaledVector(odeSystem->state, &odeSystem->state[dim], dt, dim);
		addScaledVector(odeSystem->state, a1,  dt*dt*2.0f/3.0f, dim);
		addScaledVector(odeSystem->state, a0, -dt*dt*1.0f/6.0f, dim);
		position = false;
	} else {
        // calculate the new acceleration at point x(t+h):
		odeSystem->calculateAccelerations(derivatives);
		addScaledVector(&odeSystem->state[dim], derivatives, dt*5.0f/12.0f, dim);
		addScaledVector(&odeSystem->state[dim], a1, dt*2.0f/3.0f, dim);
		addScaledVector(&odeSystem->state[dim], a0, -dt*1.0f/12.0f, dim);
		// remap the arrays: a0 <-- a1; a1 <-- derivatives;
		float *temp;
		temp = a0;
		a0 = a1;
		a1 = derivatives;
		derivatives = temp;

		position = true;
	}
}

// deferred feature
void Solver::performStep() {}
